/*
 * File name: laserplugin.cc
 * Date:      Fri Sep 21 2018 22:17:55 GMT+0200 (CEST) 
 * Author:    Miroslav Kulich (kulich@cvut.cz)
 */


#include "laserplugin.h"
#include "gazebo/physics/physics.hh"

using namespace imr;

GZ_REGISTER_SENSOR_PLUGIN(CLaserPlugin)

const int NUM_SCANS_PER_SECOND=10;


//CLaserPlugin constructor 
CLaserPlugin::CLaserPlugin() {
  
};


CLaserPlugin::~CLaserPlugin() {
  sensor.reset();
  world.reset();
}

robot_messages::LaserCfgMsg laserSettings;

bool settingsService(robot_messages::LaserCfgMsg &out) {
  // std::cout << "Setting " << laserSettings.scansamplecount() << std::endl;
  out = laserSettings;
  return true;
}


void CLaserPlugin::Load(sensors::SensorPtr _parent, sdf::ElementPtr /*_sdf*/)
{
  topicName = "/turtlebot";
  sensor = std::dynamic_pointer_cast<sensors::RaySensor>(_parent);

  if (!sensor)
    gzthrow("RayPlugin requires a Ray Sensor as its parent");

  // sensor = parentSensor->LaserShape();
   world = physics::get_world(sensor->WorldName());
  // node->Init(parentSensor->WorldName());
  connection = sensor->LaserShape()->ConnectNewLaserScans(std::bind(&CLaserPlugin::OnNewLaserScans, this));

  ignition::transport::AdvertiseMessageOptions opts;
  opts.SetMsgsPerSec(NUM_SCANS_PER_SECOND);

  publisher = node.Advertise<robot_messages::LaserScanMsg>("/turtlebot/laser",opts);
  if (!publisher)
  {
    std::cerr << "Error advertising topic [" << "/turtlebot/laser" << "]" << std::endl;
  }

  if (!node.Advertise("/turtlebot/laser_configuration_request", settingsService))
  {
    std::cerr << "Error advertising service [" << "/turtlebot/laser_configuration_request" << "]" << std::endl;
  }

  laserSettings.set_index(0);//TODO: magic constant
  laserSettings.set_maxrange(sensor->RangeMax());
  laserSettings.set_angularresolution(sensor->AngleResolution());
  laserSettings.set_minangle(sensor->AngleMin().Radian());
  laserSettings.set_scansamplecount(sensor->RayCount());
  laserSettings.set_frequency(NUM_SCANS_PER_SECOND);
  laserSettings.mutable_position()->set_x(0);
  laserSettings.mutable_position()->set_y(0);
	laserSettings.mutable_position()->set_z(0);


// laserSettings.set_angle_max(sensor->AngleMax().Radian());
  // laserSettings.set_range_min(sensor->RangeMin());
  

  gzmsg << sensor->AngleMin() << " " << sensor->AngleMax() << " " << sensor->RayCount() << "    " << sensor->IsActive() << " " << sensor->RangeMax() <<  std::endl;
  gzmsg << ("LaserPlugin loaded.") << std::endl;
}


void CLaserPlugin::OnNewLaserScans()
{
  double max = sensor->RangeMax();
  // gzdbg << "Got laser data" << std::endl;
  std::vector<double>	ranges;
  sensor->Ranges(ranges);
  robot_messages::LaserScanMsg msg;
  msg.set_index(1); //TODO:magic constant
  msg.clear_range();
  for(auto r: ranges) {
     msg.add_range(r<max ? r : max);
  }
  publisher.Publish(msg);
}

/* end of laserplugin.cc */
