/*
 * File name: plugin.cc
 * Date:      Wed Sep 19 2018 14:24:33 GMT+0200 (CEST) 
 * Author:    Miroslav Kulich
 */

#include "plugin.h"
#include <ignition/math/Vector2.hh>
// #include <functional>

using namespace imr;

// void TurtleBotPlugin::Init() {
//   std::cout << "TourtleBotPlugin initialized." << std::endl;
// }

const int NUM_POSES_PER_SECOND=10;

void TurtleBotPlugin::Load(gazebo::physics::ModelPtr parent, sdf::ElementPtr sdf) {
  std::string topicName = "/turtlebot";
  index = 0;
  model = parent;
  updateConnection = gazebo::event::Events::ConnectWorldUpdateBegin(std::bind(&TurtleBotPlugin::OnUpdate, this));

  // Create the command subscriber 
//  velocityConnection = inNode->Subscribe("/velocity", &TurtleBotPlugin::OnVelocityMessage, this);
  ignition::transport::AdvertiseMessageOptions opts;
  opts.SetMsgsPerSec(NUM_POSES_PER_SECOND);
  publisher = inNode.Advertise<robot_messages::OdometryMsg>(topicName + "/odometry");
  if (!publisher)
  {
    std::cerr << "Error advertising topic [" << topicName + "/odometry" << "]" << std::endl;
  }

  bool res =  inNode.Subscribe(topicName + "/control_velocities", &TurtleBotPlugin::OnVelocityMessage, this);
  std::cout << "substribed: " << (res ? "yes" : "no") << std::endl;
  gzmsg << ("TurtleBot plugin loaded.") << std::endl;

}

void TurtleBotPlugin::OnVelocityMessage(const robot_messages::MotionControlMsg &msg) {
  std::cout << "Velocity received: " <<  msg.forwardvelocity() << "  angular: "<< msg.angularvelocity() << std::endl;
  model->SetLinearVel(ignition::math::Vector3d(msg.forwardvelocity(), msg.angularvelocity(), 0));
}

void TurtleBotPlugin::OnUpdate() {
  static int i = 0;
  if (++i == 100) {
	  gazebo::physics::ModelState state = gazebo::physics::ModelState(model);
	  ignition::math::Pose3d position = state.Pose();
	  auto pos = position.Pos();
	  auto rot = position.Rot();	

  robot_messages::OdometryMsg msg;
  msg.set_index(index);;
  msg.set_x(pos.X());
  msg.set_y(pos.Y());
  msg.set_heading(rot.Yaw());
  // std::cout << "Publishing " << pos.X() << " " << pos.Y() << " " << rot.Yaw() << std::endl; 
  publisher.Publish(msg);

    i = 0;
  }
}


// Register this plugin with the simulator
GZ_REGISTER_MODEL_PLUGIN(TurtleBotPlugin)
